//genesis
/**********************************************************************
** This demo is      
**                  copyright (C) 1998 Upinder S. Bhalla.
** It is made available under the terms of the GNU General Public License. 
** See the file COPYRIGHT for the full notice.
**********************************************************************/
// This demo file replicates figure 2C.
// Our model file includes a lot of pathways other than the one in
// figure 2A, but they are all downstream of the activity we are
// looking at here, so we'll delete them.
// Our total PLC-gamma activity has two contributions, from the
// Ca-bound form and also the Ca-bound as well as tyr phosph form.
// So we estimate the total activity by rate of production of IP3.
// Note that we need to get rid of PLC-beta to accomplish this.
// Despite deleting a lot of extraneous stuff, this simulation takes
// a long time (nearly an hour on a Pentium II 266) because of the
// time needed for each concentration step to reach equilibrium

include allmapk_egfr127_plots.g

delete /kinetics/PLCbeta/PLC // The IP3 degradation pathway is in here
                             // so we can't delete the whole thing.
delete /kinetics/PKC
delete /kinetics/PLA2
delete /kinetics/Gq

// Update the display to eliminate the stuff that has been deleted
call /edit/draw/tree RESET
// Show the additional plots which include IP3 and DAG
xshow /moregraphs

setclock 0 0.002
setclock 1 0.002
setclock 2 10
setclock 3 10


function do_stim(ca, egf)
	float ca, egf

	setfield /kinetics/EGFR/EGF CoInit {egf} slave_enable 4
	setfield /kinetics/Ca CoInit {ca} slave_enable 4
	// Need to use a fine timestep to get over the Ca transient 
	setclock 0 1e-5
	setclock 1 1e-5
	step 2 -t
	// Back to the usual timestep
	setclock 0 0.002
	setclock 1 0.002

	step 998 -t

// Having run this model, I know that the max IP3 is 1.374 uM, so I
// can get activity on a scale from 0 to 1 by simply dividing.
	echo {ca}"	"{egf}"	"{getfield /kinetics/IP3 Co}"		"{{getfield /kinetics/IP3 Co}/1.374}
// Also send the output to a file named /tmp/fig2C.tab
	echo {ca}"	"{egf}"	"{getfield /kinetics/IP3 Co}"		"{{getfield /kinetics/IP3 Co}/1.374} >> /tmp/fig2C.tab

end

reset

echo "Doing simulation for PLC-gamma activity against Ca at 0 uM EGF"
echo "Ca	EGF	PLC-gamma	Scaled PLC-gamma"
echo "Ca	EGF	PLC-gamma	Scaled PLC-gamma" >> /tmp/fig2C.tab
do_stim 0	0
do_stim 0.01	0
do_stim 0.03	0
do_stim 0.05	0
do_stim 0.08	0
do_stim 0.1	0
do_stim 0.2	0
do_stim 0.5	0
do_stim 1	0
do_stim 2	0
do_stim 5	0
do_stim 10	0
do_stim 100	0

reset
echo "Doing simulation for PLC-gamma activity against Ca at 0.1 uM EGF"
echo "Ca	EGF	PLC-gamma	Scaled PLC-gamma"
do_stim 0	0.1
do_stim 0.01	0.1
do_stim 0.03	0.1
do_stim 0.05	0.1
do_stim 0.08	0.1
do_stim 0.1	0.1
do_stim 0.2	0.1
do_stim 0.5	0.1
do_stim 1	0.1
do_stim 2	0.1
do_stim 5	0.1
do_stim 10	0.1
do_stim 100	0.1

