//genesis
/**********************************************************************
** This demo is      
**                  copyright (C) 1998 Upinder S. Bhalla.
** It is made available under the terms of the GNU General Public License. 
** See the file COPYRIGHT for the full notice.
**********************************************************************/
// This demo replicates figure 2H. It is basically the same as
// figure 2G, except that it explores a wider range of MKP-1 
// times and levels, and generates text output to indicate whether
// or not the turnoff occurred. The idea here is to bracket the
// range of MKP-1 added to cause turnoff.
// Obviously, this demo takes forever. Allow 20 min per data point
// on a PII-266.
// Output is saved in /tmp/fig2H.table

include allmapk125_plots.g

float fastclock = 5e-5
float normclock = 2e-3

function get_tot_MKP1
	float tot = {getfield /kinetics/MKP-1 Co}
	tot = tot + {getfield /kinetics/MKP-1/MKP1-tyr-deph CoComplex}
	tot = tot + {getfield /kinetics/MKP-1/MKP1-thr-deph CoComplex}
	return {tot}
end

function do_stim(time, mkp1)
	float time
	float mkp1

	float amount_added
	float temp

	reset
	setfield /kinetics/Ca CoInit 0.08 slave_enable 4
	// Push the system into the active state
	setfield /kinetics/AA CoInit 50 slave_enable 4
	setclock 0 {fastclock}
	setclock 1 {fastclock}
	step 2 -t
	setclock 0 {normclock}
	setclock 1 {normclock}
	step 998 -t

	// Let the system relax to steady-state
	setfield /kinetics/AA CoInit 0.0 slave_enable 0
	step 2000 -t

	// apply more MKP-1 so that total is our desired level
	// Note that buffering MKP-1 does not produce the desired
	// result, as a significant amount of the phosphatase is complexed
	// when MAPK is activated

	amount_added = mkp1
	setfield /kinetics/MKP-1 \
		Co {{getfield /kinetics/MKP-1 Co} + amount_added}
	setclock 0 {fastclock}
	setclock 1 {fastclock}
	step 2 -t
	setclock 0 {normclock}
	setclock 1 {normclock}
	step {time -2} -t

	setclock 0 {fastclock}
	setclock 1 {fastclock}
	// Remove amount_added. We can't do this all at once, because
	// this may give -ve concentrations. So we need to iterate it out
	temp = {getfield /kinetics/MKP-1 Co}
	while (temp < amount_added)
		amount_added = amount_added -temp
		setfield /kinetics/MKP-1 Co 0
		step 1 -t
		temp = {getfield /kinetics/MKP-1 Co}
	end
	setfield /kinetics/MKP-1 Co {temp - amount_added}
	step 1 -t

	setclock 0 {normclock}
	setclock 1 {normclock}
	step 3600 -t

	if ({getfield /kinetics/PKC-active Co} > 0.3)
		echo {time/60}"		"{mkp1}"		No"
		echo {time/60}"		"{mkp1}"		No" >> /tmp/fig2H.table
	else
		echo {time/60}"		"{mkp1}"		Yes"
		echo {time/60}"		"{mkp1}"		Yes" >> /tmp/fig2H.table
	end
end

echo "Time(min)	[MKP1](uM)	Turnoff ?"
echo "Time(min)	[MKP1](uM)	Turnoff ?" >> /tmp/fig2H.table

do_stim	300	10

do_stim	480	0.3
do_stim	480	0.28

do_stim 510	0.18
do_stim 510	0.16

do_stim 540	0.08
do_stim 540	0.06

do_stim 600	0.02
do_stim 600	0.016

do_stim 1200	0.0052
do_stim 1200	0.0048

do_stim 2400	0.0036
do_stim 2400	0.0032

do_stim 7200	0.0032
do_stim 7200	0.0028

do_stim 14400	0.0028
do_stim 14400	0.0024
