//genesis
/**********************************************************************
** This demo is      
**                  copyright (C) 1998 Upinder S. Bhalla.
** It is made available under the terms of the GNU General Public License. 
** See the file COPYRIGHT for the full notice.
**********************************************************************/

// this demo produces all the plots in figure 3. It is 
// complicated for two reasons. First, the very sharp Ca
// transients introduce numerical instability unless we use a
// very fine timestep for the transient. Second, we use a 
// rather complicated stimlus pattern which needs to be coordinated
// with a table of Ca levels.

float fastclock = 0.00005
float normclock = 0.002 // need to lower this for numerical stability
float fastplotclock = 0.1
float normplotclock = 10
float fastequil = 2
float equiltime = 4000
float tet_time = 1
float inter_tet_time = 600
float post_tet_time = 4000
float steady_PKC = 0.09878 // from other simulations
int ntet = 3
float table_lead_time = 1.0 // time lag between start of table and
					// start of tetanus
int fix_PKA = 0

include allcam36_demo.g
xshow /moregraphs
setfield /graphs/conc# overlay 1
setfield /moregraphs/conc# overlay 1
setclock 2 {normplotclock}


function do_tetanus(ntet)
	int ntet
	int i

	setfield /kinetics/PKC-active CoInit {steady_PKC} slave_enable 4

	reset
	
	// equiltime sec of equil
	setclock 0 {fastclock}
	setclock 1 {fastclock}
	setclock 3 {fastplotclock}
	step {fastequil} -t
	setclock 0 {normclock}
	setclock 1 {normclock}
	setclock 3 {normplotclock}
	step {equiltime - fastequil} -t

	if ({fix_PKA}) // Fix PKA activity at steady-state level
		setfield /kinetics/PKA-active \
			CoInit {getfield /kinetics/PKA-active Co} \
			slave_enable 4
	end


	// tet_time sec of tetanic stim
	setclock 0 {fastclock}
	setclock 1 {fastclock}
	setclock 3 {fastplotclock}
	step {tet_time} -t

	for (i = 1 ; i < ntet; i = i + 1)
		// inter_tet_time between tetani
		step {fastequil} -t
		setclock 0 {normclock}
		setclock 1 {normclock}
		setclock 3 {normplotclock}
		step {inter_tet_time - fastequil - tet_time} -t
	
		// tet_time sec of tetanic stim
		setclock 0 {fastclock}
		setclock 1 {fastclock}
		setclock 3 {fastplotclock}
		step {tet_time} -t
	end

	// post_tet_time sec of waiting after tetanus
	step {fastequil} -t
	setclock 0 {normclock}
	setclock 1 {normclock}
	setclock 3 {normplotclock}
	step {post_tet_time - fastequil} -t
end

reset

// Set up Ca influx from tabulated values
// The Ca concentrations are in the file abstr2_ca.gmn,
// which is derived from a compartmental simulation of Ca influx
// into a dendritic spine on a CA1 neuron.
table_lead_time = 1
file2tab abstr2_ca.gmn /kinetics/Ca_tab table -xy 16000
setfield /kinetics/Ca_tab \
	table->xmin 0 \
	table->xmax 138.5 \
	input {table_lead_time - equiltime}
do_tetanus {ntet}
do_save_all_plots fig3.plt

fix_PKA = 1

do_tetanus {ntet}
do_save_all_plots fig3_fix_PKA.plt
