//genesis
/**********************************************************************
** This demo is      
**                  copyright (C) 1998 Upinder S. Bhalla.
** It is made available under the terms of the GNU General Public License. 
** See the file COPYRIGHT for the full notice.
**********************************************************************/

// This file replicates all the sections in figure 4.
// Some notes:
// 1. It is big and ugly, because it has to coordinate multiple
// timesteps for stability, as well as reading in a Ca influx
// table generated from a compartmental cell model, as well
// as delivering a series of synaptic inputs for each tetanus.
// 2. It only takes about 30 minutes to run, though, on a PII-266
// 3. We break the feedback loop in the second run using the flag
// fix_AA to buffer AA at its steady-state level just before
// applying the stimulus.
// 4. It dumps its graphical output to the files fig4.plot and
//	fig4_nofb.plot.
// 5. The level of total active CaMKII as in the paper is calculated
// using the approximation that we can simply add the activities of
// the Ca-autonomous form without CaM and of the CaM-bound form.

float fastclock = 0.00005
float normclock = 0.002 // need to lower this for numerical stability
float fastplotclock = 0.1
float normplotclock = 10
float fastequil = 2
float equiltime = 4000
float tet_time = 1
float inter_tet_time = 20
float post_tet_time = 4000
int ntet = 3
float tet_freq = 100.0
float table_lead_time = 1.0 // time lag between start of table and
					// start of tetanus
float glu_width = 0.002 // 2 msec is used to approx the time of
						// presence of glutamate following a stim
int fix_AA = 0

include all125_plotstim_b.g
xshow /moregraphs

setclock 2 {normplotclock}

function do_tetanus(ntet,freq)
	int ntet
	float freq
	int i

	setfield /kinetics/AA \
		CoInit 0 \
		slave_enable 0

	reset
	
	// equiltime sec of equil
	setclock 0 {fastclock}
	setclock 1 {fastclock}
	setclock 3 {fastplotclock}
	step {fastequil} -t
	setclock 0 {normclock}
	setclock 1 {normclock}
	setclock 3 {normplotclock}
	step {equiltime - fastequil} -t

	if (fix_AA)
		setfield /kinetics/AA \
			CoInit {getfield /kinetics/AA Co} \
			slave_enable 4
	end

	// tet_time sec of tetanic stim
	setfield /kinetics/stim trig_mode 0 \
		width1 {glu_width} delay1 {1/freq - glu_width}
	setclock 0 {fastclock}
	setclock 1 {fastclock}
	setclock 3 {fastplotclock}
	step {tet_time} -t
	setfield /kinetics/stim trig_mode 1 \
		width1 {glu_width} delay1 {1/freq - glu_width}

	for (i = 1 ; i < ntet; i = i + 1)
		// inter_tet_time between tetani
		step {fastequil} -t
		setclock 0 {normclock}
		setclock 1 {normclock}
		setclock 3 {normplotclock}
		step {inter_tet_time - fastequil - tet_time} -t
	
		// tet_time sec of tetanic stim
		setfield /kinetics/stim trig_mode 0 \
			width1 {glu_width} delay1 {1/freq - glu_width}
		setclock 0 {fastclock}
		setclock 1 {fastclock}
		setclock 3 {fastplotclock}
		step {tet_time} -t
		setfield /kinetics/stim trig_mode 1 \
			width1 {glu_width} delay1 {1/freq - glu_width}
	end

	// post_tet_time sec of waiting after tetanus
	step {fastequil} -t
	setclock 0 {normclock}
	setclock 1 {normclock}
	setclock 3 {normplotclock}
	step {post_tet_time - fastequil} -t
end

setfield /kinetics/stim trig_mode 1

reset

// Set up tetanic stimulus
ntet = 3
inter_tet_time = 600

file2tab abstr2_ca.gmn /kinetics/Ca_tab table -xy 16000
setfield /kinetics/Ca_tab \
	table->xmin 0 \
	table->xmax 138.5 \
	input {table_lead_time - equiltime}
do_tetanus {ntet}	{tet_freq}
do_save_all_plots fig4.plot

fix_AA = 1
setfield /graphs/conc# overlay 1
setfield /moregraphs/conc# overlay 1

do_tetanus {ntet}	{tet_freq}
do_save_all_plots fig4_nofb.plot
